"""Compatibility for Quantity."""

__all__: list[str] = []

from plum import conversion_method, dispatch

from .api import ustrip
from .base import AbstractQuantity
from .core import Quantity
from .fast import UncheckedQuantity
from unxt._src.dimensions import AbstractDimension
from unxt._src.units.api import AbstractUnits

# ===================================================================
# Get dimensions


@dispatch  # type: ignore[misc]
def dimension_of(obj: AbstractQuantity, /) -> AbstractDimension:
    """Return the dimension of a quantity.

    Examples
    --------
    >>> from unxt import dimension_of, Quantity
    >>> q = Quantity(1, "m")
    >>> dimension_of(q)
    PhysicalType('length')

    """
    return dimension_of(obj.unit)


# ===================================================================
# Get units


@dispatch  # type: ignore[misc]
def unit_of(obj: AbstractQuantity, /) -> AbstractUnits:
    """Return the units of an object.

    Examples
    --------
    >>> from unxt import unit_of, Quantity
    >>> q = Quantity(1, "m")
    >>> unit_of(q)
    Unit("m")

    """
    return obj.unit


#####################################################################
# Conversion


@conversion_method(type_from=AbstractQuantity, type_to=UncheckedQuantity)  # type: ignore[misc]
def _quantity_to_unchecked(q: AbstractQuantity, /) -> UncheckedQuantity:
    """Convert any quantity to an unchecked quantity.

    Examples
    --------
    >>> from plum import convert
    >>> from unxt.quantity import Quantity, UncheckedQuantity

    >>> q = Quantity(1, "m")
    >>> convert(q, UncheckedQuantity)
    UncheckedQuantity(Array(1, dtype=int32, weak_type=True), unit='m')

    The self-conversion doesn't copy the object:

    >>> q = UncheckedQuantity(1, "m")
    >>> convert(q, UncheckedQuantity) is q
    True

    """
    if isinstance(q, UncheckedQuantity):
        return q
    unit = unit_of(q)
    return UncheckedQuantity(ustrip(unit, q), unit)


@conversion_method(type_from=AbstractQuantity, type_to=Quantity)  # type: ignore[misc]
def _quantity_to_checked(q: AbstractQuantity, /) -> Quantity:
    """Convert any quantity to a checked quantity.

    Examples
    --------
    >>> from plum import convert
    >>> from unxt.quantity import Quantity, UncheckedQuantity

    >>> q = UncheckedQuantity(1, "m")
    >>> q
    UncheckedQuantity(Array(1, dtype=int32, ...), unit='m')

    >>> convert(q, Quantity)
    Quantity['length'](Array(1, dtype=int32, ...), unit='m')

    The self-conversion doesn't copy the object:

    >>> q = Quantity(1, "m")
    >>> convert(q, Quantity) is q
    True

    """
    if isinstance(q, Quantity):
        return q
    u = unit_of(q)
    return Quantity(ustrip(u, q), u)



"""Quantities in JAX.

This module provides a way to work with quantities in JAX.

The main features are:

- ``unxt.quantity.Quantity``: A class representing a quantity with units.
- ``unxt.quantity.uconvert``: A function to convert a quantity to a different
  unit.
- ``unxt.quantity.ustrip``: A function to strip the units from a quantity.

"""
# ruff:noqa: F403

from ._src.quantity.api import is_unit_convertible, uconvert, ustrip
from ._src.quantity.base import AbstractQuantity
from ._src.quantity.base_parametric import AbstractParametricQuantity
from ._src.quantity.core import Quantity
from ._src.quantity.fast import UncheckedQuantity

# isort: split
# Register dispatches and conversions
from ._src.quantity import compat, functional, register_dispatches, register_primitives

__all__: list[str] = [
    # Core
    "Quantity",
    # Base
    "AbstractQuantity",
    # Fast
    "UncheckedQuantity",
    # Base Parametric
    "AbstractParametricQuantity",
    # Functional
    "uconvert",
    "ustrip",
    "is_unit_convertible",
]


# Clean up namespace
del compat, functional, register_dispatches, register_primitives



# pylint: disable=import-error

"""unxt: Quantities in JAX.

Copyright (c) 2023 Galactic Dynamics. All rights reserved.
"""

__all__: list[str] = []

from typing import TypeAlias

import astropy.units as apyu

Unit: TypeAlias = apyu.Unit | apyu.UnitBase | apyu.CompositeUnit



"""Dimensions module.

This is the private implementation of the dimensions module.

"""

__all__ = ["AbstractDimension", "dimension", "dimension_of"]

from .core import AbstractDimension, dimension, dimension_of



"""Units objects in JAX.

Copyright (c) 2023 Galactic Dynamics. All rights reserved.
"""

__all__ = [
    "Unit",  # legacy
    "unit",
    "unit_of",
]

from typing import Any

import astropy.units as apyu
from astropy.units import Unit
from plum import dispatch

from .api import AbstractUnits
from unxt._src.dimensions import AbstractDimension

# ===================================================================
# Construct units


@dispatch
def unit(obj: AbstractUnits, /) -> AbstractUnits:
    """Construct the units from a units object.

    Examples
    --------
    >>> import unxt as u
    >>> m = u.unit("m")

    >>> u.unit(m) is m
    True

    """
    return obj


@dispatch  # type: ignore[no-redef]
def unit(obj: str, /) -> AbstractUnits:
    """Construct units from a string.

    Examples
    --------
    >>> import unxt as u
    >>> m = u.unit("m")
    >>> m
    Unit("m")

    """
    return apyu.Unit(obj)


# ===================================================================
# Get units


@dispatch.abstract
def unit_of(obj: Any, /) -> AbstractUnits:
    """Return the units of an object."""


@dispatch
def unit_of(obj: Any, /) -> None:
    """Return the units of an object.

    Examples
    --------
    >>> import unxt as u
    >>> print(u.unit_of(1))
    None

    """
    return None  # noqa: RET501


@dispatch
def unit_of(obj: AbstractUnits, /) -> AbstractUnits:
    """Return the units of an unit.

    Examples
    --------
    >>> import unxt as u
    >>> m = u.unit("m")

    >>> u.unit_of(m)
    Unit("m")

    """
    return obj


# ===================================================================
# Get dimensions


@dispatch  # type: ignore[misc]
def dimension_of(obj: AbstractUnits, /) -> AbstractDimension:
    """Return the dimensions of the given units.

    Examples
    --------
    >>> import unxt as u
    >>> u.dimension_of(u.unit("km"))
    PhysicalType('length')

    """
    return obj.physical_type
